/** We have 3 CPUs in use. 
1st CPU is placed outside the house on the door.
2nd CPU is used to detected someone at the door (after unnecessary few steps, like domino falling).
3rd CPU is placed inside the house with the house owner. 
In our explanation we will refer to them as 1st, 2nd and 3rd CPUs. 

Steps:
1.  Someone pulls a string by the door. The 2nd CPU senses light change and sends a message to 1st CPU using MQTT (through 
    adafruit.io) that someone is at the door. 
    How is the message sent? 
    2nd CPU publishes a light value to a light feed which 1st CPU is subscribed to.
2.  1st CPU changes the value of virtual pin VO to 2 on Blynk. Using a Blynk webhook, the change of V0 value to 2 triggers a 
    make.com scenario.
3.  A line is added to google sheets with time of the new visits and adds 1 to a counter counting all visits. 
4.  1st CPU lets 3rd CPU know someone is at the door using Blynk and make.com. 
    How? 
    The new counter value is retrieved from google sheets, then it is send using an HTTP post request to the address of 
    the 3rd CPU webhook.
5.  Using Make.com a random happy gif is added to a Facebook post on the house owner’s Facebook page, posting
    “YAY! I have a new visitor” with the chosen gif. 
6.  The HTTP request (step 4) using Blynk webhook, triggers an alarm noise on 3rd CPU. The noise alerts the house owner 
    someone is at the door.
7.  On the house owner blink’s app it says “Wow, u have had X visitors this month!” and average rate of accepted entrances
8.  When it is 7pm virtual pin VO is changed to 1 (1st CPU code). Then, using make.com scenario and router we ask 
    chat gpt to come up with a funny sentence in Spanish to cheer up the person. 
    Then using a TextCortexAI translator (free version of google translate), the sentence is translated to English.
9.  The sentence is sent to the house owner’s email account through gmail (using Make.com), saying 
    “Hey Buddy, I know it has been a long day but I just wanted to say: {the translated sentence} :)”. 
10. 3rd CPU senses temperature in the house and sends it to the 1st CPU. How? 
3rd CPU sets virtual pin V2 value to the detected temperature. The 1st CPU is updated on V2 value using a webhook 
    (V2 callback function is on the 1st CPU code).
11. If temp is below 16 Celsius degrees, the house owner’s blynk app says “Don't forget a jacket!”, 
    if it’s between 16-30 it says “Weather is wonderful inside!” and if it’s above 30 it says “It's boiling inside. T shirts only!”. 
12. If house owner presses right button on 3rd CPU, 1st CPU shows green lights and says “please go in”. 
    Else, house owner presses left button on 3rd CPU 1st CPU shows red lights and says “sorry you can not go in”. 
    how does it work? 
    By changing value of virtual pin V1 on 3rd CPU code, the new value is updated on 3rd CPU (V1 callback function is on 1st CPU code). 
    If V1==1 entrance allowed, if V1==0 entrance denied. 

This code is the 3rd CPU code.

  The circuit:
    No components attached to input/output.
    Sensors used- light.
    Uses Blynk, make.com, adafruit.io (mqtt).

  Video link: https://drive.google.com/file/d/1NGyTcrLBZ12IYkXyvyJryrFrds027Fxj/view?usp=sharing
  Created By: 
    Shira Cohen ID 318971827
    Nadav Poran ID 206933954
    Kylie Koshet ID 206862708

**/
#include <Adafruit_CircuitPlayground.h>
#define TINY_GSM_MODEM_ESP8266

#define SerialMon Serial
#define SerialAT Serial1
#define MQTT_USERNAME "sshiracccc"
#define MQTT_PASSWORD "aio_zgZX127Kfh0HQEXUd1lWoSO9XO5q"

const char MQTT_CLIENTID[] = MQTT_USERNAME __DATE__ __TIME__;

// WiFi connection credentials
const char wifiSSID[] = "wireless";
const char wifiPass[] = "5101051010";

// MQTT details
const char* broker = "io.adafruit.com";

const char lightTopic[] = MQTT_USERNAME "/feeds/light";

#include <TinyGsmClient.h>
#include <PubSubClient.h>

TinyGsm modem(SerialAT);
TinyGsmClient client(modem);
PubSubClient mqtt(client);

uint32_t lastReconnectAttempt = 0;

// inialization of variables
char light[50];
String lightStr;
long lastPublished = 0;

void setup() {
  //  mqtt set up
  CircuitPlayground.begin();
  SerialMon.begin(115200);
  delay(1000);
  SerialAT.begin(115200);
  SerialMon.println("Initializing modem...");
  modem.restart();
  String modemInfo = modem.getModemInfo();
  SerialMon.print("Modem Info: ");
  SerialMon.println(modemInfo);

  SerialMon.print(F("Setting SSID/password..."));
  if (!modem.networkConnect(wifiSSID, wifiPass)) {
    SerialMon.println(" fail");
    while(true);
  }
  SerialMon.println(" success");

  SerialMon.print("Waiting for network...");
  if (!modem.waitForNetwork()) {
    SerialMon.println(" fail");
    while(true);
  }
  SerialMon.println(" success");

  if (modem.isNetworkConnected()) {
    SerialMon.println("Network connected");
  }
  mqtt.setServer(broker, 1883);  
}

void loop() {
  if (!mqtt.connected()) {
    uint32_t t = millis();
    if (t - lastReconnectAttempt > 10000L) {
      lastReconnectAttempt = t;
      if (mqttConnect()) {
        lastReconnectAttempt = 0;
      }
    }
    delay(100);
    return;
  }
  mqtt.loop();
  // disconnects occur otherwise
  delay(1000);

  int lightlevel = CircuitPlayground.lightSensor();

  // update occures only if there is change in light level (string was pulled)
  if (millis() - lastPublished > 10000 && lightlevel>350) {
    Serial.println("publishing light");
    float flight = CircuitPlayground.lightSensor();
    lightStr = String(flight);
    lightStr.toCharArray(light, lightStr.length() + 1);
    // updates detected light level to the light feed on adafruit.io
    mqtt.publish(lightTopic, light);
    lastPublished = millis();
  }
}

// connecting mqtt
boolean mqttConnect() {
  SerialMon.print("Connecting to ");
  SerialMon.print(broker);
  // no password? no problem
  // boolean status = mqtt.connect(MQTT_CLIENTID);
  boolean status = mqtt.connect(MQTT_CLIENTID, MQTT_USERNAME, MQTT_PASSWORD);
  if (status == false) {
    SerialMon.println(" fail");
    return false;
  }
  SerialMon.println(" success");
  return mqtt.connected();
}
